package core

import (
	"encoding/json"
	"fmt"
	"strconv"
	"testing"
)

func TestQueryTask(t *testing.T) {

	tasks, err := QueryAllTask()

	if err != nil {
		t.Fatal(err)
	}

	for k, v := range tasks {

		t.Log(k, "v.User", v.User, "v.TaskType", v.TaskType, "v.TaskId", v.TaskId)
	}

}

//GetTasks()

func TestAddTasks(t *testing.T) {
	cfg := Config{
		ApiKey:            "u5HOlaBhMFNqXbs7lznEuUQVx",
		ApiKeySecrect:     "VhpTsl4TJUQi9FSAymajDCWfpgyJoK2d18i4lX9sPGiWc462nR",
		AccessToken:       "1783145144700874752-F38iX5PVmnOe3RHKwB0mc8tj4vObaT",
		AccessTokenSecret: "O8BhSezZA8M8Jlz0rZJReBsV9HZO420iWdU58wCG2BXaA",
		Token:             "AAAAAAAAAAAAAAAAAAAAAIZTwQEAAAAApSMHaI24HWCEVYV0mx%2F2HauZHI0%3D6GrtIbJbiAwGJr4rZIz4WdrTLO6qI4zRGN4ZKpaOqUbwRRauPI",
	}
	task := TaskInDB{
		User:          "1783145144700874752",
		TaskType:      "tweet_liking_users",
		TaskId:        "1800805503066661056",
		ApiConfig:     parseToApiConfig(cfg),
		Start:         true,
		Stop:          true,
		FollowerCount: 0,
	}
	res, _, err := DBClient.From("tasks").Insert(task, true, "", "representation", "").Execute()
	if err != nil {
		t.Fatal(err)
	}
	t.Log(res)
}

func TestGetTaskInDB(t *testing.T) {
	data, count, err := DBClient.From("tasks").Select("*", "exact", false).
		Eq("start", "true").Eq("stop", "true").
		Execute()

	if err != nil {
		t.Fatal(err)
	}

	fmt.Println("found count", count)

	res := make([]TaskInDB, 0, count)

	if err := json.Unmarshal(data, &res); err != nil {
		t.Fatal(err)
	}

	for k, v := range res {
		t.Log(k, "v.User", v.User, "v.TaskType", v.TaskType, "v.TaskId", v.TaskId, "v.Config", v.ApiConfig)
	}
}

func TestGetTasks(t *testing.T) {

	tasks, err := GetTasks()

	if err != nil {
		t.Fatal(err)
	}

	for k, v := range tasks {

		t.Log(k, "v.User", v.UserId, "v.TaskType", v.TaskType, "v.TaskId", v.TaskId, "Token", v.Token, "accessToken", v.AccessToken)
	}

	//exist, err := CheckTaskExist("1783145144700874752", "aon_aonet", "followers")
	//if err != nil {
	//	t.Fatal(err)
	//}
	//t.Log("exist", exist)

}

// func TestInsertTask(t *testing.T) {

// 	req := Req{
// 		Project: "1",
// 		TweetId: "11",
// 	}

// 	err := Insert(req)
// 	if err != nil {
// 		t.Fatal(err)
// 	}

// 	// fmt.Println("found", ok, task)
// }

// func TestTwitterAccountFromDB(t *testing.T) {

// 	accounts, err := TwitterAccountFromDB()
// 	if err != nil {
// 		t.Fatal(err)
// 	}

// 	for k, v := range accounts {
// 		t.Log(k, v.User, v.PassWd)
// 	}

// 	// fmt.Println("found", ok, task)
// }

func TestUpdateTask(t *testing.T) {
	aonConfig := Config{
		ApiKey:            "u5HOlaBhMFNqXbs7lznEuUQVx",
		ApiKeySecrect:     "VhpTsl4TJUQi9FSAymajDCWfpgyJoK2d18i4lX9sPGiWc462nR",
		AccessToken:       "1783145144700874752-jS6Ow8fuXIJl2M6Ao9IuNwfNMpkM2r",
		AccessTokenSecret: "NDgp8oQ72ulhjdv2Xs4PzzKOozq36bBQBKjAquUwe00pT",
		Token:             "AAAAAAAAAAAAAAAAAAAAAIZTwQEAAAAA4PxqcVAC4Wl2JYGHm4i%2Fcalh93o%3DDhSWWmvsHjj2XGInA3ZvYtz9C91nQ9YD1jbFQFJUHjK3niU4X1",
	}
	appBaseConfig := Config{
		ApiKey:            "UsNE66ZLht4EZVyu8K1eTKiN9",
		ApiKeySecrect:     "dUJ5CBW8NfrNy6QiAV6BPRY0q860yA018fBL6djgkaOcDB2RGf",
		AccessToken:       "1657936763577581569-9iWT0MTlRyXk6fVWvmJEFAxirL0N9d",
		AccessTokenSecret: "p0sEpaSN4ZETDrXrzpdn9155s5ZNrzKHovyIUbHiI9tZj",
		Token:             "AAAAAAAAAAAAAAAAAAAAAPI%2BwwEAAAAAiMNc1Uy0oN1KMP8jKH28DmZZRBc%3DcsfwVvWi6vgIVIgw2Auz4SUhayv84GJA7jsIVNAaLoTOarJB5x",
	}
	tasks, err := QueryAllTask()
	if err != nil {
		t.Fatal(err)
	}
	for _, task := range tasks {
		switch task.TaskId {
		case "aon_aonet", "1800805503066661056", "1843181642300674228":
			task.ApiConfig = parseToApiConfig(aonConfig)
		case "1853676550111306025", "AppBaseGlobal":
			task.ApiConfig = parseToApiConfig(appBaseConfig)
		}
		idstr := strconv.Itoa(task.ID)
		t.Log("id str ", idstr, "apiconfig", task.ApiConfig)
		// update task.
		_, _, err := DBClient.From("tasks").Update(&task, "", "exact").Eq("id", idstr).Execute()
		if err != nil {
			t.Error("update task error:", err)
		}
	}

}
