package core

import (
	"container/list"
	"log/slog"
	"time"

	twitter "github.com/g8rswimmer/go-twitter/v2"
)

/*

处理三种情况：

1. idx 为空；
2. idx 正常匹配
3. idx 没有匹配上
x 4. 到最后也没匹配上，或者超过了多少页。 外面处理；

*/

type Idx struct {
	idx []UserTask
	//newIdx []UserTask
	//taskId string
	List *list.List
}

func NewIdx(i []UserTask) *Idx {
	return &Idx{
		idx: i,
		//newIdx: make([]UserTask, 0, 10),
		List: list.New(),
	}
}

func (s *Idx) Idx(page []UserTask) (bool, *list.List) {
	if len(page) == 0 {
		return false, nil
	}

	if s.idx != nil && len(s.idx) == 0 {

		newList := list.New()

		for _, v := range page {
			newList.PushFront(v)
		}

		s.idx = page

		return true, newList
	}

	newItems := list.New()
	for k, v := range page {
		match := false
		for ik, iv := range s.idx {
			if v.UserId == iv.UserId {
				//slog.Info("Idx", "pageIndex", k, "pageUserId", v.UserId, "idxIndex", ik, "idxUserId", iv.UserId)
				match = true
				break
				_, _ = k, ik
			}
		}
		if !match {
			newItems.PushBack(v)
		} else {
			// 这打开break是一个加快比较的方法，但是有些api返回的数据中包含置顶的几个用户，会导致误判，所以先开放为全部比较.
			// stop, all new items has added to newItems.
			//break
		}
	}
	if newItems.Len() < len(page) {
		// 连续, 更新 s.idx, 将新元素加入到 s.List中, 并将s.List 返回.
		s.idx = page
		for e := newItems.Front(); e != nil; e = e.Next() {
			s.List.PushFront(e.Value)
		}
		newlist := s.List
		s.List = list.New()
		return true, newlist
	} else {
		// 不连续, 不必更新 idx, 将所有新增元素加入到s.List中.
		for e := newItems.Front(); e != nil; e = e.Next() {
			s.List.PushFront(e.Value)
		}
		return false, nil
	}
}

type PageUsers struct {
	idx *Idx
}

func (p *PageUsers) GetIdx() *Idx {
	return p.idx
}

func NewPageUsers(idx *Idx) *PageUsers {

	return &PageUsers{
		idx: idx,
	}
}

type req func(tweetId string, id string, next string) ([]*twitter.UserObj, string, *twitter.RateLimit, error)

/*
TODO 最后一页的标识，没有处理；
*/
func (p *PageUsers) Request(tweetId string, id string, next string, f req) ([]UserTask, error) {
	users, newNext, rt, err := f(tweetId, id, next)
	if err != nil {
		return nil, err
	}
	slog.Info("Request", "tweet id", tweetId, "next", next, "newnext", newNext, "len(users)", len(users))

	taskUser := UserObjectToUserTask(users)

	// 查询最新的一页 followers, 去历史中匹配。
	// 如果已经与历史记录相连接，表明已经完成了查询，不必再继续查next.
	ok, l := p.idx.Idx(taskUser)
	if ok {
		res := make([]UserTask, 0, l.Len())
		for e := l.Front(); e != nil; e = e.Next() {
			if user, ok := e.Value.(UserTask); ok {
				res = append(res, user)
			}
		}
		return res, nil
	}
	slog.Info("Task to be continued", "tweet id", tweetId, "next", newNext)
	// Check if we need to respect rate limits
	if rt != nil && rt.Remaining <= 0 {
		// Wait until the rate limit resets (plus a small buffer)
		waitTime := time.Until(rt.Reset.Time().Add(500 * time.Millisecond))
		time.Sleep(waitTime)
	}
	time.Sleep(2 * time.Second)

	return p.Request(tweetId, id, newNext, f)
}

func UserObjectToUserTask(pageUsers []*twitter.UserObj) []UserTask {

	res := make([]UserTask, 0, len(pageUsers))

	for _, v := range pageUsers {
		res = append(res, UserTask{
			//TaskId   string `json:"task_id"`
			UserId:   v.ID,       //string `json:"user_id"`
			UserName: v.UserName, //string `json:"user_name"`
		})
	}

	return res
}
